<?php

add_action('rest_api_init', 'alianspay_callback_url');

/**
 * Use to define the callback url route 
 * and to call the appropriate function
 * @return void
 */

function alianspay_callback_url(){
    
    $alianspay_gateway = new Alianspay_Woocommerce_Gateway();
    
    //get the end of the callback url choosen by the admin
    $end_of_callback_url = explode('callback/', $alianspay_gateway->get_callback_url());

   
  
    // defines the route
    register_rest_route( 'callback', $end_of_callback_url[1] ,array(
        'methods'  => 'POST',
        'callback' => 'alianspay_update_order_status',
        'permission_callback' => '__return_true'
    ));
    
}


/**
 * Change a order status after the payment has been performed 
 * 
 * @param $request
 * @return array
 */

function alianspay_update_order_status($request) {
    Alianspay_Woocommerce_Gateway::log('begin callback');
    $parameters = $request->get_body();
    $data= '';
    //get the data according to the content-type of the request 
    if(str_contains($_SERVER["CONTENT_TYPE"], 'application/json')){
        Alianspay_Woocommerce_Gateway::log("traitement1 ");
        $data = json_decode($parameters, true);
    }else{
        Alianspay_Woocommerce_Gateway::log("traitement2 ");
        parse_str($parameters, $data);
    }

    $order = wc_get_order($data["Reference"]);
    Alianspay_Woocommerce_Gateway::log("Callback result ref: ".$data["Reference"]." status: ".$data["Reference"]);
    if($order){
        
        if( $data["Status"] === 'SUCCESSFUL'){
            // Payment complete
            $order->update_status( 'completed', __( 'successful payment', Alianspay_Woocommerce_Gateway::ALIANSPAY_GATEWAY_ID) );
            $order->add_order_note( sprintf( __( "Le paiement s'est bien pass\E9: %1$s", 'woocommerce' ), $data["Status"] ) );
            //update_post_meta( $order->get_id(), '_orangemoney_status', $data["transaction_status"] );
           update_post_meta( $order->get_id(), '_alianspaystatus', 'AliansPay ' );
           update_post_meta( $order->get_id(), '_transaction_id', $data["PaymentMethod"]." - ".$data["PaymentAccount"] );
            $result = [
                "status" => "success",
                "message" => "Order updated",
                "transaction_status" => "success"
            ];
            
        }else if($data["Status"] === 'CANCELED'){
            
            $order->update_status( 'cancelled', 
            __( 'payment cancelled', Alianspay_Woocommerce_Gateway::ALIANSPAY_GATEWAY_ID) );

            $result = [
                "status" => "success",
                "message" => "Order updated",
                "transaction_status" => "cancelled"
            ];

        }else if($data["Status"] === 'FAILED'){
            
            $order->update_status( 'failed', 
            __( 'failed payment', Alianspay_Woocommerce_Gateway::ALIANSPAY_GATEWAY_ID) );
            
            $result = [
                "status" => "success",
                "message" => "Order updated",
                "transaction_status" => "failed"
            ];
        }

    }else{
        $result = [
            "status" => "error",
            "message" => "Order not found"
        ];
    }
   
    $response= new WP_REST_Response($result);
    $response->set_status(200);
    return $response;
}

/**
 * check the signature of the request
 *
 * @param array $data
 * @param string $private_key
 * @return boolean
 */
function alianspay_check_signature($data, $private_key){
    $signature_received = $data["signature"];
    
    $new_signature = $data["transaction_ref"].$data["transaction_type"].$data["transaction_amount"].
                     $data["transaction_currency"].$data["transaction_operator"].$private_key;
    $new_md5_signature = md5($new_signature);

    if($signature_received === $new_md5_signature){
        return true;
    }else{
        return false;
    }
}


/**
 * add data received to the log file
 *
 * @param array $data
 * @return void
 */
function alianspay_add_log_data($data){
    // add data in the wordpress log file 
    if (!function_exists('write_log')) {
        function write_log ( $log )  {
            if ( true === WP_DEBUG ) {
                if ( is_array( $log ) || is_object( $log ) ) {
                    error_log( print_r( $log, true ) );
                } else {
                    error_log( $log );
                }
            }
        }
    }
    write_log($data);
}